/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ElmoTaxonomyFinder.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.elmo.impl;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import it.tidalwave.openrdf.elmo.ElmoFinder;
import it.tidalwave.bluebill.taxonomy.Finder;
import it.tidalwave.bluebill.taxonomy.TaxonomyConcept;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ElmoTaxonomyFinder.java,v 31ee4e8a62bc 2010/07/17 14:44:24 fabrizio $
 *
 **********************************************************************************************************************/
public class ElmoTaxonomyFinder<T> extends ElmoFinder<T> implements Finder<T>
  {
    private static final String CLASS = ElmoTaxonomyFinder.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private static final Map<Class<?>, Class<?>> IMPLEMENTATION_MAP = new HashMap<Class<?>, Class<?>>();

    protected String name;

    static
      {
        IMPLEMENTATION_MAP.put(TaxonomyConcept.class, ElmoTaxonomyConcept.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected ElmoTaxonomyFinder (@Nonnull final Class<T> entityClass)
      {
        super(entityClass,
              (Class<T>)IMPLEMENTATION_MAP.get(entityClass),
              "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
              "PREFIX skos: <http://www.w3.org/2004/02/skos/core#>\n");
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    @Override
    public Finder<T> name (@Nonnull final String name)
      {
        this.name = name;
        return this;
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void process (@Nonnull Set<String> auxStatements)
      {
        if (name != null)
          {
            auxStatements.add(String.format("?%s rdfs:label '%s'", entityName, name));
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected it.tidalwave.util.Finder<T> createClone()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }
